@@ -1,35 +1,36 @@
-
+from typing import Any, Dict, List, Sequence, Union
 
 import numpy as np
 import torch
 import torch.nn as nn
 
+DETECTED_OUTPUT_TYPES = Union[Sequence[Any], Dict[Any, torch.Tensor], torch.Tensor]
 
 
 class LayerInfo:
     """ Class that holds information about a layer module. """
 
-    def __init__(self, module, depth, depth_index):
+    def __init__(self, module: nn.Module, depth: int, depth_index: int):
         # Identifying information
         self.layer_id = id(module)
         self.module = module
         self.class_name = str(module.__class__).split(".")[-1].split("'")[0]
-        self.inner_layers = {}
+        self.inner_layers = {}  # type: Dict[str, List[int]]
         self.depth = depth
         self.depth_index = depth_index
 
         # Statistics
         self.trainable = True
         self.is_recursive = False
-        self.output_size = []
-        self.kernel_size = []
+        self.output_size = []  # type: List[Union[int, Sequence[Any], torch.Size]]
+        self.kernel_size = []  # type: List[int]
         self.num_params = 0
         self.macs = 0
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return "{}: {}-{}".format(self.class_name, self.depth, self.depth_index)
 
-    def calculate_output_size(self, outputs, batch_dim):
+    def calculate_output_size(self, outputs: DETECTED_OUTPUT_TYPES, batch_dim: int) -> None:
         """ Set output_size using the model's outputs. """
         if isinstance(outputs, (list, tuple)):
             try:
@@ -50,9 +51,11 @@ class LayerInfo:
             self.output_size[batch_dim] = -1
 
         else:
-            raise TypeError("Model contains a layer with an unsupported output type: {}".format(outputs))
+            raise TypeError(
+                "Model contains a layer with an unsupported output type: {}".format(outputs)
+            )
 
-    def calculate_num_params(self):
+    def calculate_num_params(self) -> None:
         """ Set num_params using the module's parameters.  """
         for name, param in self.module.named_parameters():
             self.num_params += param.nelement()
@@ -76,7 +79,7 @@ class LayerInfo:
                 self.inner_layers[name] = list(param.size())
                 self.macs += param.nelement()
 
-    def check_recursive(self, summary_list):
+    def check_recursive(self, summary_list: "List[LayerInfo]") -> None:
         """ if the current module is already-used, mark as (recursive).
         Must check before adding line to the summary. """
         if list(self.module.named_parameters()):
@@ -84,19 +87,19 @@ class LayerInfo:
                 if self.layer_id == other_layer.layer_id:
                     self.is_recursive = True
 
-    def macs_to_str(self, reached_max_depth):
+    def macs_to_str(self, reached_max_depth: bool) -> str:
         """ Convert MACs to string. """
         if self.num_params > 0 and (reached_max_depth or not any(self.module.children())):
             return "{:,}".format(self.macs)
         return "--"
 
-    def num_params_to_str(self, reached_max_depth = False):
+    def num_params_to_str(self, reached_max_depth: bool = False) -> str:
         """ Convert num_params to string. """
         assert self.num_params >= 0
         if self.is_recursive:
             return "(recursive)"
         if self.num_params > 0:
-            param_count_str = "{:,}".format(self.num_params)
+            param_count_str = "{:,}".format((self.num_params))
             if reached_max_depth or not any(self.module.children()):
                 if not self.trainable:
                     return "({})".format(param_count_str)
