@@ -1,4 +1,5 @@
 """ torchsummary.py """
+from typing import Any, Dict, Generator, List, Optional, Sequence, Union
 
 import torch
 import torch.nn as nn
@@ -6,24 +7,42 @@ from torch.utils.hooks import RemovableHandle
 
 from .formatting import FormattingOptions, Verbosity
 from .layer_info import LayerInfo
-from .model_statistics import ModelStatistics
+from .model_statistics import CORRECTED_INPUT_SIZE_TYPE, ModelStatistics
 
 # Some modules do the computation themselves using parameters
 # or the parameters of children. Treat these as layers.
 LAYER_MODULES = (torch.nn.MultiheadAttention,)
+INPUT_SIZE_TYPE = Sequence[Union[int, Sequence[Any], torch.Size]]
 
-def summary(model, input_data, *args, batch_dim = 0, branching = True, col_names = ("output_size", "num_params"), col_width = 25, depth = 3, device = None, dtypes = None, verbose = 1, **kwargs):
+
+def summary(
+    model: nn.Module,
+    input_data: Union[torch.Tensor, torch.Size, Sequence[torch.Tensor], INPUT_SIZE_TYPE],
+    *args: Any,
+    batch_dim: int = 0,
+    branching: bool = True,
+    col_names: Sequence[str] = ("output_size", "num_params"),
+    col_width: int = 25,
+    depth: int = 3,
+    device: Optional[torch.device] = None,
+    dtypes: Optional[List[torch.dtype]] = None,
+    verbose: int = 1,
+    **kwargs: Any
+) -> ModelStatistics:
     """
     Summarize the given PyTorch model. Summarized information includes:
-        1) output shape, 2) kernel shape, 3) number of the parameters
+        1) output shape,
+        2) kernel shape,
+        3) number of the parameters
         4) operations (Mult-Adds)
-        
+
     Arguments:
         model (nn.Module): PyTorch model to summarize
         input_data (Sequence of Sizes or Tensors):
             Example input tensor of the model (dtypes inferred from model input).
             - OR -
-            Shape of input data as a List/Tuple/torch.Size (dtypes must match model input, default is FloatTensors).
+            Shape of input data as a List/Tuple/torch.Size (dtypes must match model
+            input, default is FloatTensors).
         batch_dim (int): batch_dimension of input data
         branching (bool): Whether to use the branching layout for the printed output.
         col_names (Sequence[str]): specify which columns to show in the output.
@@ -42,9 +61,9 @@ def summary(model, input_data, *args, batch_dim = 0, branching = True, col_names
         args, kwargs: Other arguments used in `model.forward` function.
     """
     assert verbose in (0, 1, 2)
-    summary_list = []
-    hooks = []
-    idx = {}
+    summary_list = []  # type: List[LayerInfo]
+    hooks = []  # type: List[RemovableHandle]
+    idx = {}  # type: Dict[int, int]
     apply_hooks(model, model, depth, summary_list, hooks, idx, batch_dim)
 
     if device is None:
@@ -66,7 +85,8 @@ def summary(model, input_data, *args, batch_dim = 0, branching = True, col_names
             x = get_input_tensor(input_size, batch_dim, dtypes, device)
 
     else:
-        raise TypeError("Input type is not recognized. Please ensure input_data is valid.\n"
+        raise TypeError(
+            "Input type is not recognized. Please ensure input_data is valid.\n"
             "For multiple inputs to the network, ensure input_data passed in is "
             "a sequence of tensors or a list of tuple sizes. If you are having trouble here, "
             "please submit a GitHub issue."
@@ -79,7 +99,9 @@ def summary(model, input_data, *args, batch_dim = 0, branching = True, col_names
         with torch.no_grad():
             _ = model.to(device)(*x, *args, **kwargs)
     except Exception:
-        print("Failed to run torchsummary, printing sizes of executed layers: {}".format(summary_list))
+        print(
+            "Failed to run torchsummary, printing sizes of executed layers: {}".format(summary_list)
+        )
         raise
     finally:
         for hook in hooks:
@@ -93,7 +115,12 @@ def summary(model, input_data, *args, batch_dim = 0, branching = True, col_names
     return results
 
 
-def get_input_tensor(input_size, batch_dim, dtypes, device):
+def get_input_tensor(
+    input_size: CORRECTED_INPUT_SIZE_TYPE,
+    batch_dim: int,
+    dtypes: List[torch.dtype],
+    device: torch.device,
+) -> List[torch.Tensor]:
     """ Get input_tensor with batch size 2 for use in model.forward() """
     x = []
     for size, dtype in zip(input_size, dtypes):
@@ -109,11 +136,11 @@ def get_input_tensor(input_size, batch_dim, dtypes, device):
     return x
 
 
-def get_correct_input_sizes(input_size):
+def get_correct_input_sizes(input_size: INPUT_SIZE_TYPE) -> CORRECTED_INPUT_SIZE_TYPE:
     """ Convert input_size to the correct form, which is a list of tuples.
     Also handles multiple inputs to the network. """
 
-    def flatten(nested_array):
+    def flatten(nested_array: INPUT_SIZE_TYPE) -> Generator:
         """ Flattens a nested array. """
         for item in nested_array:
             if isinstance(item, (list, tuple)):
@@ -133,10 +160,19 @@ def get_correct_input_sizes(input_size):
     return [input_size]
 
 
-def apply_hooks(module, orig_model, depth, summary_list, hooks, idx, batch_dim, curr_depth = 0):
+def apply_hooks(
+    module: nn.Module,
+    orig_model: nn.Module,
+    depth: int,
+    summary_list: List[LayerInfo],
+    hooks: List[RemovableHandle],
+    idx: Dict[int, int],
+    batch_dim: int,
+    curr_depth: int = 0,
+) -> None:
     """ Recursively adds hooks to all layers of the model. """
 
-    def hook(module, inputs, outputs):
+    def hook(module: nn.Module, inputs: Any, outputs: Any) -> None:
         """ Create a LayerInfo object to aggregate information about that layer. """
         del inputs
         idx[curr_depth] = idx.get(curr_depth, 0) + 1
@@ -153,5 +189,6 @@ def apply_hooks(module, orig_model, depth, summary_list, hooks, idx, batch_dim,
 
     if curr_depth <= depth:
         for child in module.children():
-            apply_hooks(child, orig_model, depth, summary_list, hooks, idx, batch_dim, curr_depth + 1
+            apply_hooks(
+                child, orig_model, depth, summary_list, hooks, idx, batch_dim, curr_depth + 1
             )
